package	jp.co.sds_corp.syslog.viewer;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import jp.co.sds_corp.syslog.filter.IFilter;
import jp.co.sds_corp.syslog.viewer.filter.IFilterPanel;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.window.ValuePanel;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* {@link IFilter}を設定するダイアログです。
*/
public class FilterDialog extends JDialog
{
	private static final long serialVersionUID = 1L;
	static CmnLogger logger = CmnLogger.getInstance();
	SyslogParam	param;
	ValuePanel	VAL_TYPE;
	Box			btnBox;
	JButton		BTN_OK = new JButton( "OK" );
	JButton		BTN_CANCEL = new JButton( "Cancel" );
	IFilterPanel	fltPanel;
	boolean		bOk;
	ResourceBundle res;

	IFilter		filter;

	public FilterDialog( JFrame parent, SyslogParam param ) {
		super( parent );
		this.param = param;
		createControls();
	}

	public FilterDialog( JDialog parent, SyslogParam param ) {
		super( parent );
		this.param = param;
		createControls();
	}

	protected void createControls() {
		setModal( true );
		setResizable( false );
		WindowUtility.setHideOnEscape( this );

		res = ResourceBundle.getBundle( "resource.viewer" );
		String text;
		text = res.getString( "filter.title" );
		setTitle( text );
		setSize( 335, 260 );
		setFont( param.getDefaultFont() );
		getContentPane().setLayout(
			new BoxLayout( getContentPane(), BoxLayout.Y_AXIS ));

		WindowUtility.setHideOnEscape( this );

		btnBox = new Box( BoxLayout.X_AXIS );
		btnBox.add( BTN_OK );
		btnBox.add( BTN_CANCEL );

		BTN_OK.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onOk();
			}
		});
		BTN_CANCEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onCancel();
			}
		});

	}

	public IFilter getFilter() {
		return filter;
	}

	public void setFilter( IFilter filter ) {
		this.filter = filter;
	}

	public void clearFilter() {
		this.filter = null;
	}

	public void setVisible( boolean var ) {
		if ( !var ) {
			super.setVisible( false );
			return;
		}
		bOk = false;
		getContentPane().removeAll();
		String msg, title;
		try {
			Box box = new Box( BoxLayout.Y_AXIS );
			fltPanel = param.getFilterPanel( filter );
			if ( fltPanel == null ) {
				title = res.getString( "filter.err2.title" );
				msg = res.getString( "filter.err2.message" );
				JOptionPane.showMessageDialog( this,
					msg, title,
					JOptionPane.ERROR_MESSAGE );
				return;
			}
			fltPanel.setFilter( filter );
			box.add( (JPanel )fltPanel );
			box.add( btnBox );
			getContentPane().add( box );
			BTN_OK.requestFocus();
			getRootPane().setDefaultButton( BTN_OK );
			SwingUtilities.updateComponentTreeUI( this );
			super.setVisible( true );
		} catch ( Exception ex ) {
			logger.error( this, "error", ex );
			title = res.getString( "filter.err1.title" );
			msg = res.getString( "filter.err1.message" );
			JOptionPane.showMessageDialog( this,
					msg + "\n" + ex, title,
					JOptionPane.ERROR_MESSAGE );
		}
	}

	public void onOk() {
		try {
			fltPanel.updateFilter();
		} catch ( Exception ex ) {
			logger.error( this, "update error " + ex );
			String title = res.getString( "filter.err3.title" );
			String msg = res.getString( "filter.err3.message" );
			if ( ex.getMessage() == null ) {
				msg += "\n" + ex.toString();
			} else {
				msg += "\n" + ex.getMessage();
			}
			JOptionPane.showMessageDialog( this,
				msg, title,
				JOptionPane.ERROR_MESSAGE );
			return;
		}
		bOk = true;
		setVisible( false );
	}

	public void onCancel() {
		setVisible( false );
	}

	public boolean isOk() {
		return bOk;
	}
}
