package	jp.co.sds_corp.syslog.viewer;

import java.awt.Component;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import jp.co.sds_corp.syslog.filter.IFilter;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* {@link IFilter}を設定するパネルです。
*/
public class FilterPanel extends JPanel
{
	private static final long serialVersionUID = 1L;
	static CmnLogger logger = CmnLogger.getInstance();
	Window		owner;
	SyslogParam	param;
	Vector<IFilter>		vecFilter = new Vector<IFilter>();
	FilterModel	model;
	JTable	table;
	JButton	BTN_ADD;
	JButton	BTN_EDIT;
	JButton	BTN_DEL;
	JButton	BTN_VAL;
	FilterDialog	dlg;
	boolean		bChg = false;
	ResourceBundle res;

	public FilterPanel( JDialog owner, SyslogParam param ) {
		this.owner = owner;
		this.param = param;
		dlg = new FilterDialog( owner, param );
		createControls();
	}

	public FilterPanel( JFrame owner, SyslogParam param ) {
		this.owner = owner;
		this.param = param;
		dlg = new FilterDialog( owner, param );
		createControls();
	}

	protected void createControls() {
		setLayout( new BoxLayout( this, BoxLayout.X_AXIS ));

		res = ResourceBundle.getBundle( "resource.viewer" );
		String text;
		model = new FilterModel();
		table = new JTable( model );
		WindowUtility.removeTableEscape( table );
		JScrollPane sp = new JScrollPane( table );
		add( sp );
		table.addMouseListener( new MouseAdapter() {
        	public void mouseClicked( MouseEvent e ) {
				if ( e.getClickCount() == 2 ) {
					onEdit();
				}
			}
		});
		table.addKeyListener( new KeyAdapter() {
			public void keyPressed( KeyEvent e ) {
				if ( e.getKeyChar() == '\n' ) {
					onEdit();
					e.consume();
				}
			}
		});
		table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		TableColumn col;
		col = table.getColumnModel().getColumn( 1 );
		col.setCellRenderer( new DescRenderer() );

		JPanel pnl = new JPanel();
		pnl.setLayout( new BoxLayout( pnl, BoxLayout.Y_AXIS ));

		text = res.getString( "filter.add.text" );
		BTN_ADD = new JButton( text );
		BTN_ADD.setMnemonic( KeyEvent.VK_A );
		pnl.add( BTN_ADD );
		text = res.getString( "filter.edit.text" );
		BTN_EDIT = new JButton( text );
		BTN_EDIT.setMnemonic( KeyEvent.VK_E );
		pnl.add( BTN_EDIT );
		text = res.getString( "filter.del.text" );
		BTN_DEL = new JButton( text );
		BTN_DEL.setMnemonic( KeyEvent.VK_D );
		pnl.add( BTN_DEL );
		text = res.getString( "filter.validbtn.text" );
		BTN_VAL = new JButton( text );
		BTN_VAL.setMnemonic( KeyEvent.VK_V );
		pnl.add( BTN_VAL );
		add( pnl );
		BTN_ADD.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onAdd();
			}
		});
		BTN_EDIT.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onEdit();
			}
		});
		BTN_DEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onDel();
			}
		});
		BTN_VAL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onValid();
			}
		});

	}

	public void addFilterList( java.util.List<IFilter> lstFilter ) {
		vecFilter.addAll( lstFilter );
		model.fireTableDataChanged();
	}
	public void addFilter( IFilter filter ) {
		vecFilter.add( filter );
		int nRow = vecFilter.size() - 1;
		model.fireTableRowsInserted( nRow, nRow );
	}
	public void clearFilter() {
		bChg = false;
		vecFilter.clear();
		table.clearSelection();
		model.fireTableDataChanged();
	}
	public java.util.List<IFilter> listFilters() {
		return vecFilter;
	}

	protected void onAdd() {
		IFilter f = createFilter();
		if ( f == null ) {
			return;
		}
		dlg.clearFilter();
		dlg.setFilter( f );
		WindowUtility.center( this, dlg );
		dlg.setVisible( true );
		if ( dlg.isOk() ) {
			IFilter filter = dlg.getFilter();
			vecFilter.add( filter );
			int nRow = vecFilter.size() - 1;
			model.fireTableRowsInserted( nRow, nRow );
			bChg = true;
		}
	}

	private IFilter createFilter() {
		Map<String,SyslogParam.Registry> mapFilter = param.getFilterMap();
		String[] options = new String[ mapFilter.size() ];
		int nPos = 0;
		HashMap<String,SyslogParam.Registry> mapSel = new HashMap<String,SyslogParam.Registry>();
		Iterator<String> itr = mapFilter.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			SyslogParam.Registry reg = (SyslogParam.Registry )mapFilter.get( key );
			mapSel.put( reg.getType(), reg );
			options[nPos++] = reg.getType();
		}
		String msg, title;
		title = res.getString( "filter.add.title" );
		msg = res.getString( "filter.add.message" );
		String sel = (String )JOptionPane.showInputDialog( owner,
			msg, title,
			JOptionPane.INFORMATION_MESSAGE,
			null, options, options[0] );
		if ( sel == null ) {
			return null;
		}

		String cls = ((SyslogParam.Registry )mapSel.get( sel )).getOrgClass();
		IFilter ret = null;
		try {
			ret = (IFilter )Class.forName( cls ).newInstance();
		} catch ( Exception ex ) {
			logger.error( this, "create error", ex );
			title = res.getString( "filter.add.err1.title" );
			msg = res.getString( "filter.add.err1.message" );
			JOptionPane.showMessageDialog( this,
				msg + " [" + cls + "]\n" + ex, title,
				JOptionPane.ERROR_MESSAGE );
			return null;
		}
		return ret;
	}

	protected void onEdit() {
		int nPos = table.getSelectedRow();
		if ( table.getRowCount() == 0 || nPos < 0 ) {
			return;
		}
		IFilter filter = (IFilter )vecFilter.get( nPos );
		dlg.setFilter( filter );
		WindowUtility.center( this, dlg );
		dlg.setVisible( true );
		if ( dlg.isOk() ) {
			bChg = true;
			model.fireTableRowsUpdated( nPos, nPos );
		}
	}

	protected void onDel() {
		int nPos = table.getSelectedRow();
		if ( table.getRowCount() == 0 || nPos < 0 ) {
			return;
		}
		String msg, title;
		title = res.getString( "filter.del.title" );
		msg = res.getString( "filter.del.message" );
		int nRet = JOptionPane.showConfirmDialog( this,
			msg, title,
			JOptionPane.YES_NO_OPTION );
		if ( nRet != JOptionPane.YES_OPTION ) {
			return;
		}
		vecFilter.remove( nPos );
		model.fireTableRowsDeleted( nPos, nPos );
		table.clearSelection();
		bChg = true;
	}

	protected void onValid() {
		int nPos = table.getSelectedRow();
		if ( table.getRowCount() == 0 || nPos < 0 ) {
			return;
		}
		IFilter filter = (IFilter )vecFilter.get( nPos );
		filter.setValid( !filter.isValid() );
		model.fireTableRowsUpdated( nPos, nPos );
		bChg = true;
	}

	public void setChange( boolean var ) {
		this.bChg = var;
	}
	public boolean isChange() {
		return bChg;
	}

	class FilterModel extends AbstractTableModel
	{
		private static final long serialVersionUID = 1L;
		String[] cols = { "Type", "Option" };

		public int getColumnCount() {
			return cols.length;
		}
		public String getColumnName( int col ) {
			return cols[col];
		}

		public int getRowCount() {
			return vecFilter.size();
		}
		public Object getValueAt( int row, int col ) {
			IFilter filter = (IFilter )vecFilter.get( row );
			switch ( col ) {
			case 0:
				SyslogParam.Registry reg = param.getFilterType( filter );
				if ( reg == null ) {
					return "(UNKNOWN)";
				}
				return reg.getType();
			case 1: return filter.paramString();
			}
			return null;
		}
	}

	public static class DescRenderer extends DefaultTableCellRenderer
	{
		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent(
			JTable table, Object obj, boolean isSelected, boolean hasFocus,
			int row, int column)
		{
			JLabel lbl = (JLabel )super.getTableCellRendererComponent(
							table, obj, isSelected, hasFocus, row, column );
			lbl.setToolTipText( obj.toString() );
			return lbl;
		}
	}

}

