package	jp.co.sds_corp.syslog.viewer;

import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;

import javax.swing.Icon;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.util.syslog.SyslogMsg;

/**
* syslog一覧を表示するテーブルです。
*/
public class SyslogTable extends JTable
{
	private static final long serialVersionUID = 1L;
	SyslogParam	param;
	SyslogModel	model;

	public SyslogTable( SyslogModel model, SyslogParam param ) {
		super( model );
		this.model = model;
		this.param = param;

		setColumnSelectionAllowed( false );
		setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		setAutoResizeMode( JTable.AUTO_RESIZE_LAST_COLUMN );

		setIconColumn();
		setRowHeight( param.getRowHeight() );

		int[] nWidth = param.getColumnWidth();
		if ( nWidth != null ) {
			for ( int nLoop = 0; nLoop < param.getColumnCount(); nLoop++ ) {
				TableColumn column = null;
				column = getColumnModel().getColumn(nLoop);
				column.setPreferredWidth( nWidth[nLoop] );
			}
		}
	}

	public void setIconColumn() {
		if ( param == null ) {
			return;
		}
		int nPos = param.getColumnPos( "Icon" );
		if ( nPos < 0 ) {
			return;
		}
		TableColumn col = getColumnModel().getColumn( nPos );
		col.setCellRenderer( new SyslogCellRenderer() );
		int height = calcRowHeight() + 10;
		col.setMaxWidth( height );
		col.setMinWidth( height );
		col.setPreferredWidth( height );
	}

	public void setFont( Font font ) {
		super.setFont( font );
		getTableHeader().setFont( font );
		int height = calcRowHeight();
		setRowHeight( height );
		setIconColumn();
	}

	private int calcRowHeight() {
		int ret = getRowHeight();
		Font font = getFont();
		Graphics2D grf = (Graphics2D )getGraphics();
		if ( grf == null ) {
			return ret;
		}
		FontRenderContext frc = grf.getFontRenderContext();
		Rectangle2D rect = font.getMaxCharBounds( frc );
		ret = (int )rect.getHeight();
		ret = (int )font.getLineMetrics( "Aあ", frc ).getHeight();
		return ret;
	}

	class SyslogCellRenderer extends DefaultTableCellRenderer
	{
		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent( JTable tbl, Object obj,
					boolean isSelected, boolean hasFocus, int row, int clm )
		{
			SyslogEvent ev = (SyslogEvent )model.getData( row );
			SyslogMsg msg = ev.getMessage();
			int height = calcRowHeight();
			Icon i = param.getIcon( msg, height + 2 );
			setIcon( i );
			return this;
		}
	}

}
